[PATCH i-g-t v7 2/3] lib/igt_kms: loosen duplicate check in igt_display_refresh

Jessica Zhang quic_jesszhan at quicinc.com
Sat Feb 15 00:15:52 UTC 2025


From: Esha Bharadwaj <quic_ebharadw at quicinc.com>

Change the duplicate check in igt_display_refresh() so it allows for
pipes to be shared by encoders that are valid clones of each other.

Signed-off-by: Esha Bharadwaj <quic_ebharadw at quicinc.com>
Signed-off-by: Jessica Zhang <quic_jesszhan at quicinc.com>
---
Changes in v7:
- Added documentation for igt_output_is_valid_clone() (Kamil)
---
 lib/igt_kms.c | 61 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++---
 lib/igt_kms.h |  1 +
 2 files changed, 59 insertions(+), 3 deletions(-)

diff --git a/lib/igt_kms.h b/lib/igt_kms.h
index 55c24149a..933882273 100644
--- a/lib/igt_kms.h
+++ b/lib/igt_kms.h
@@ -554,6 +554,7 @@ igt_plane_t *igt_output_get_plane_type_index(igt_output_t *output,
 					     int plane_type, int index);
 igt_output_t *igt_output_from_connector(igt_display_t *display,
     drmModeConnector *connector);
+bool igt_output_is_valid_clone(int drm_fd, igt_output_t *target, igt_output_t *clone);
 void igt_output_refresh(igt_output_t *output);
 drmModeModeInfo *igt_std_1024_mode_get(int vrefresh);
 void igt_output_set_writeback_fb(igt_output_t *output, struct igt_fb *fb);

diff --git a/lib/igt_kms.c b/lib/igt_kms.c
index ab35a11f8..83ffb3da6 100644
--- a/lib/igt_kms.c
+++ b/lib/igt_kms.c
@@ -3341,6 +3341,60 @@ int kmstest_get_encoder_idx(drmModeRes *resources, drmModeEncoder *encoder)
 		     encoder->encoder_id, resources->count_encoders - 1);
 }
 
+/**
+ * igt_output_is_valid_clone:
+ * @drm_fd: A drm file descriptor
+ * @target: Target output
+ * @clone: Cloned output
+ *
+ * Checks if the cloned output is listed as a possible clone in the target
+ * output's encoder
+ *
+ * Returns: True if the cloned output is a valid clone of the target output,
+ * else false
+ */
+bool igt_output_is_valid_clone(int drm_fd, igt_output_t *target, igt_output_t *clone)
+{
+	drmModeEncoder *target_enc;
+	drmModeEncoder *clone_enc;
+	drmModeRes *resources = drmModeGetResources(drm_fd);
+	uint32_t clone_mask;
+
+	if (!target || !clone)
+		return false;
+
+	target_enc = target->config.encoder;
+	clone_enc = clone->config.encoder;
+
+	if (!target_enc || !clone_enc)
+		return false;
+
+	clone_mask = 1 << kmstest_get_encoder_idx(resources, clone_enc);
+
+	return ((target_enc->possible_clones & clone_mask) == clone_mask);
+}
+
+static bool igt_output_has_duplicates(igt_display_t *display, igt_output_t *output,
+				      unsigned long pipes_in_use, int start_idx)
+{
+	igt_output_t *clone_output;
+
+	for (int j = start_idx + 1; j < display->n_outputs; j++) {
+		clone_output = &display->outputs[j];
+
+		/*
+		 * Check that any encoders with duplicated pipes
+		 * are possible clones of each other. If they
+		 * aren't, report the pipe as duplicated
+		 */
+		if ((pipes_in_use & (1 << output->pending_pipe)) &&
+		    !igt_output_is_valid_clone(display->drm_fd, output, clone_output))
+			return true;
+	}
+
+	return false;
+}
+
 static void igt_display_refresh(igt_display_t *display)
 {
 	igt_output_t *output;
@@ -3348,13 +3402,14 @@ static void igt_display_refresh(igt_display_t *display)
 
 	unsigned long pipes_in_use = 0;
 
        /* Check that two outputs aren't trying to use the same pipe */
 	for (i = 0; i < display->n_outputs; i++) {
 		output = &display->outputs[i];
 
 		if (output->pending_pipe != PIPE_NONE) {
-			if (pipes_in_use & (1 << output->pending_pipe))
-				goto report_dup;
+			if (igt_output_has_duplicates(display, output,
+						      pipes_in_use, i))
+					goto report_dup;
 
 			pipes_in_use |= 1 << output->pending_pipe;
 		}
@@ -3348,13 +3402,14 @@ static void igt_display_refresh(igt_display_t *display)
 	unsigned long pipes_in_use = 0;
 
-       /* Check that two outputs aren't trying to use the same pipe */
+	/* Check that two non-clone outputs aren't trying to use the same pipe */
 	for (i = 0; i < display->n_outputs; i++) {
 		output = &display->outputs[i];
-- 
2.48.1



More information about the igt-dev mailing list